@@ -27,15 +27,17 @@ first, the signature or delta is constructed of a ROPath iterator.  In
 the second, the ROPath iterator is put into tar block form.
 """
 
-import cStringIO, types
+import cStringIO, types, math
 from duplicity import statistics
 from duplicity import util
+from duplicity import globals
 from duplicity.path import * #@UnusedWildImport
 from duplicity.lazy import * #@UnusedWildImport
+from duplicity import progress
 
-# A StatsObj will be written to this from DirDelta_WriteSig only.
+# A StatsObj will be written to this from DirDelta and DirDelta_WriteSig.
 stats = None
-
+tracker = None
 
 class DiffDirException(Exception):
     pass
@@ -82,7 +84,7 @@ def DirDelta(path_iter, dirsig_fileobj_list):
     else:
         sig_iter = sigtar2path_iter(dirsig_fileobj_list)
     delta_iter = get_delta_iter(path_iter, sig_iter)
-    if globals.dry_run:
+    if globals.dry_run or (globals.progress and not progress.tracker.has_collected_evidence()):
         return DummyBlockIter(delta_iter)
     else:
         return DeltaTarBlockIter(delta_iter)
@@ -189,8 +191,10 @@ def get_delta_iter(new_iter, sig_iter, sig_fileobj=None):
         log.Debug(_("Comparing %s and %s") % (new_path and new_path.index,
                                               sig_path and sig_path.index))
         if not new_path or not new_path.type:
-            # file doesn't exist
-            if sig_path and sig_path.exists():
+            # File doesn't exist (but ignore attempts to delete base dir;
+            # old versions of duplicity could have written out the sigtar in
+            # such a way as to fool us; LP: #929067)
+            if sig_path and sig_path.exists() and sig_path.index != ():
                 # but signature says it did
                 log.Info(_("D %s") %
                          (sig_path.get_relative_path(),),
@@ -361,7 +365,7 @@ def DirDelta_WriteSig(path_iter, sig_infp_list, newsig_outfp):
     else:
         sig_path_iter = sigtar2path_iter(sig_infp_list)
     delta_iter = get_delta_iter(path_iter, sig_path_iter, newsig_outfp)
-    if globals.dry_run:
+    if globals.dry_run or (globals.progress and not progress.tracker.has_collected_evidence()):
         return DummyBlockIter(delta_iter)
     else:
         return DeltaTarBlockIter(delta_iter)
@@ -479,14 +483,14 @@ class TarBlockIter:
             filler_data = ""
         return TarBlock(index, "%s%s%s" % (headers, file_data, filler_data))
 
-    def process(self, val, size):
+    def process(self, val):
         """
         Turn next value of input_iter into a TarBlock
         """
         assert not self.process_waiting
         XXX # Override in subclass @UndefinedVariable
 
-    def process_continued(self, size):
+    def process_continued(self):
         """
         Get more tarblocks
 
@@ -496,15 +500,15 @@ class TarBlockIter:
         assert self.process_waiting
         XXX # Override in subclass @UndefinedVariable
 
-    def next(self, size = 1024 * 1024):
+    def next(self):
         """
-        Return next block, no bigger than size, and update offset
+        Return next block and update offset
         """
         if self.process_waiting:
-            result = self.process_continued(size)
+            result = self.process_continued()
         else:
             # Below a StopIteration exception will just be passed upwards
-            result = self.process(self.input_iter.next(), size)
+            result = self.process(self.input_iter.next())
         block_number = self.process_next_vol_number
         self.offset += len(result.data)
         self.previous_index = result.index
@@ -515,6 +519,13 @@ class TarBlockIter:
             self.remember_next = False
         return result
 
+    def get_read_size(self):
+        # read size must always be the same, because if we are restarting a
+        # backup volume where the previous volume ended in a data block, we
+        # have to be able to assume it's length in order to continue reading
+        # the file from the right place.
+        return 64 * 1024
+
     def get_previous_index(self):
         """
         Return index of last tarblock, or None if no previous index
@@ -551,7 +562,7 @@ class DummyBlockIter(TarBlockIter):
     """
     TarBlockIter that does no file reading
     """
-    def process(self, delta_ropath, size):
+    def process(self, delta_ropath):
         """
         Get a fake tarblock from delta_ropath
         """
@@ -575,13 +586,9 @@ class SigTarBlockIter(TarBlockIter):
     """
     TarBlockIter that yields blocks of a signature tar from path_iter
     """
-    def process(self, path, size):
+    def process(self, path):
         """
         Return associated signature TarBlock from path
-
-        Here size is just ignored --- let's hope a signature isn't too
-        big.  Also signatures are stored in multiple volumes so it
-        doesn't matter.
         """
         ti = path.get_tarinfo()
         if path.isreg():
@@ -604,7 +611,7 @@ class DeltaTarBlockIter(TarBlockIter):
     delta_path_iter, so the delta information has already been
     calculated.
     """
-    def process(self, delta_ropath, size):
+    def process(self, delta_ropath):
         """
         Get a tarblock from delta_ropath
         """
@@ -629,8 +636,7 @@ class DeltaTarBlockIter(TarBlockIter):
 
         # Now handle single volume block case
         fp = delta_ropath.open("rb")
-        # Below the 512 is the usual length of a tar header
-        data, last_block = self.get_data_block(fp, size - 512)
+        data, last_block = self.get_data_block(fp)
         if stats:
             stats.RawDeltaSize += len(data)
         if last_block:
@@ -652,11 +658,11 @@ class DeltaTarBlockIter(TarBlockIter):
         self.process_next_vol_number = 2
         return self.tarinfo2tarblock(index, ti, data)
 
-    def get_data_block(self, fp, max_size):
+    def get_data_block(self, fp):
         """
         Return pair (next data block, boolean last data block)
         """
-        read_size = min(64*1024, max(max_size, 512))
+        read_size = self.get_read_size()
         buf = fp.read(read_size)
         if len(buf) < read_size:
             if fp.close():
@@ -665,7 +671,7 @@ class DeltaTarBlockIter(TarBlockIter):
         else:
             return (buf, False)
 
-    def process_continued(self, size):
+    def process_continued(self):
         """
         Return next volume in multivol diff or snapshot
         """
@@ -673,7 +679,7 @@ class DeltaTarBlockIter(TarBlockIter):
         ropath = self.process_ropath
         ti, index = ropath.get_tarinfo(), ropath.index
         ti.name = "%s/%d" % (self.process_prefix, self.process_next_vol_number)
-        data, last_block = self.get_data_block(self.process_fp, size - 512)
+        data, last_block = self.get_data_block(self.process_fp)
         if stats:
             stats.RawDeltaSize += len(data)
         if last_block:
@@ -718,4 +724,4 @@ def get_block_size(file_len):
     else:
         # Split file into about 2000 pieces, rounding to 512
         file_blocksize = long((file_len / (2000 * 512)) * 512)
-        return min(file_blocksize, 2048L)
+        return min(file_blocksize, globals.max_blocksize)
